use crate::error::CustomError;

use super::registry::Registry;
use super::IElement;
use super::IObject;

#[derive(Debug)]
pub(crate) struct IPlainListItem {
    pub(crate) tag: Vec<IObject>,
    pub(crate) children: Vec<IElement>,
}

impl IPlainListItem {
    pub(crate) async fn new<'intermediate, 'parse>(
        registry: &mut Registry<'intermediate, 'parse>,
        original: &'intermediate organic::types::PlainListItem<'parse>,
    ) -> Result<IPlainListItem, CustomError> {
        let tag = {
            let mut ret = Vec::new();
            for obj in original.tag.iter() {
                ret.push(IObject::new(registry, obj).await?);
            }
            ret
        };

        let children = {
            let mut ret = Vec::new();
            for elem in original.children.iter() {
                ret.push(IElement::new(registry, elem).await?);
            }
            ret
        };

        Ok(IPlainListItem { tag, children })
    }
}
